<?php
/**
 * Fluid Mind EmailHandler Class for PHP5
 *
 * This class provides standard functionality for creating and sending an
 * email message. You can instantiate it and send a messaeg manually, or you
 * can make a separate class that extends this class for each email message
 * your system needs to send. It defaults to using UTF-8 as the encoding for
 * the message and all headers
 *
 * PUBLIC INTERFACES:
 * @method  setFrom($email, $name)
 * @method  setReplyTo($email, $name)
 * @method  setBounceTo($email, $name)
 * @method  setSubject($subject)
 * @method  addHeader($headerName, $content)
 * @method  setBody($body)
 * @method  setHtmlBody($htmlBody)
 * @method  addRecipient($email, $name)
 * @method  addCcRecipient($email, $name)
 * @method  addBccRecipient($email, $name)
 * @method  removeAllRecipients()
 * @method  addAttachment($filePath)
 * @method  removeAttachment($cid)
 * @method  setEncoding($encoding)
 * @method  send($email, $name)
 */
class EmailHandler
{

    // These are so self explanatory, I'm not even gonna comment them ;-)
    protected $subject = '';
    protected $from = '';
    protected $replyTo = '';
    protected $recipients = array();
    protected $ccRecipients = array();
    protected $bccRecipients = array();
    protected $bounceTo = '';
    protected $body = '';
    protected $htmlBody = '';
    protected $attachments = array();
    protected $encoding = 'UTF-8';
    protected $headers = array();
    protected $encodeBody = false;

    public $htmlLeadin =
<<<EOT
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8" />
    <style type="text/css">
    body { font-family: Times New Roman, serif; font-size: 14px; }
    </style>
</head>
<body>

EOT;

    public $htmlTrailout =
<<<EOT

</body>
</html>

EOT;

    public function __construct() {

    }


    /**
     * Specifies the email address (and optional name) that this message is sent from
     *
     * @param string $email The email address of the person/system sending this message
     * @param string $name (Optional) The name of the person/system corresponding to the given email address
     */
    public function setFrom($email = '', $name = '') {
        $this->from = $this->buildEmailAddress($email, $name);
    }


    /**
     * Specifies the email address (and optional name) that replies will be sent to
     *
     * @param string $email The email address of the person/system sending this message
     * @param string $name (Optional) The name of the person/system corresponding to the given email address
     */
    public function setReplyTo($email = '', $name = '') {
        $this->replyTo = $this->buildEmailAddress($email, $name);
    }


    /**
     * This sets the email address for bounced messages to go to
     *
     * @param string $email The email address that bounces should be sent to
     */
    public function setBounceTo($email = '') {
        mb_internal_encoding($this->encoding);
        $email = trim($email);
        if ($email != '') {
            $this->bounceTo = $email;
        }
    }


    /**
     * Sets the subject of the email message
     *
     * @param string $subject The text that should appear in the subject of the email message
     */
    public function setSubject($subject) {
        $this->subject = $subject;
    }


    /**
     * Adds a mail header.
     *
     * Not allowed to replace Subject, From, To, Cc, Bcc, X-Mailer, Mime-version, or Content-Type
     */
    public function addHeader($headerName, $content) {
        $restrictedHeaders = array('subject', 'from', 'to', 'cc', 'bcc', 'x-mailer', 'mime-version', 'content-type');

        $headerName = trim($headerName);
        if ($headerName != '' and !in_array(strtolower($headerName), $restrictedHeaders)) {
            $this->headers[$headerName] = trim($content);
        }
    }


    /**
     * Sets the plain-text version of the body of the email message
     *
     * @param string $body The text-only version of the message
     */
    public function setBody($body) {
        $this->body = $body;
    }


    /**
     * Alternate method for setBody for backward compatibility
     */
    public function setMessage($message) {
        $this->setBody($message);
    }
    

    /**
     * Sets the HTML version of the body of the email message
     *
     * @param string $htmlBody The HTML version of the message
     */
    public function setHtmlBody($htmlBody) {
        $this->htmlBody = $htmlBody;
    }


    /**
     * Alternate method for setHtmlBody for backward compatibility
     */
    public function setHtmlMessage($htmlMessage) {
        $this->setHtmlBody($htmlMessage);
    }
    

    /**
     * Adds an email address (and optional name) to the To: field
     *
     * @param string $email An email address to send this message to
     * @param string $name (Optional) The name of the person corresponding to the given email address
     */
    public function addRecipient($email = '', $name = '') {
        $this->recipients[] = $this->buildEmailAddress($email, $name);
    }


    /**
     * Adds an email address (and optional name) to the Cc: header
     *
     * @param string $email An email address to send this message to
     * @param string $name (Optional) The name of the person corresponding to the given email address
     */
    public function addCcRecipient($email = '', $name = '') {
        $this->ccRecipients[] = $this->buildEmailAddress($email, $name);
    }


    /**
     * Adds an email address (and optional name) to the Bcc: header
     *
     * @param string $email An email address to send this message to
     * @param string $name (Optional) The name of the person corresponding to the given email address
     */
    public function addBccRecipient($email = '', $name = '') {
        $this->bccRecipients[] = $this->buildEmailAddress($email, $name);
    }


    /**
     * Removes all To:, Cc:, and Bcc: recipients
     */
    public function removeAllRecipients() {
        $this->recipients = array();
        $this->ccRecipients = array();
        $this->bccRecipients = array();
    }



    /**
     * Adds a file to attach to this message.
     *
     * This method returns a unique "cid" for the attachment which can be used to include
     * it in an <img> tag within an HTML email, as such:
     *     <img src="cid:12345678" />
     *
     * @param string $filePath  The full path to the file to attach
     * @return string           The value of the Content-Id:, or false if file not found
     */
    public function addAttachment($filePath = '') {
        $filePath = trim($filePath);
        if ($filePath != '' and file_exists($filePath)) {
            $cid = uniqid();
            $this->attachments[$cid] = $filePath;
        } else {
            throw new Exception('File "' . $filePath . '" not found.');
        }

        return $cid;

    } // addAttachment


    /**
     * Sets the character encoding for the body, subject, and recipient names
     *
     * @param string $encoding  The character encoding to use for this message (default: utf-8)
     */
    public function setEncoding($encoding) {
        $encoding = trim($encoding);

        if ($encoding == '') {
            return true;
        } elseif (mb_internal_encoding($encoding)) {
            $this->encoding = $encoding;
        } else {
            throw new Exception('Unknown character encoding: "' . $encoding . '".');
        }
    }


    /**
     * Sends the email message
     *
     * If you haven't set any recipients with the addRecipient() method
     * you can specify a single recipient with this method.
     *
     * @param string $email (Optional) An email address to send this message to
     * @param string $name (Optional) The name of the person corresponding to the given email address
     * @param string $subject (Optional) The subject of the message
     */
    public function send($email = '', $name = '', $subject = '') {
        // Always scrub what has been passed in
        $email = trim($email);
        $name = trim($name);
        $subject = trim($subject);


        // Make sure we have a few required things before trying to send:
        // Who it's from, who it's going to, and (of course) what's the message?
        if ($this->from != '' and (sizeof($this->recipients) > 0 or $email != '') and ($this->body or $this->htmlBody)) {

            $recipient = array();
            if ($email != '') {
                $recipient[] = $this->buildEmailAddress($email, $name);
            }


            // Set the standard message headers
            $headers['X-Mailer'] = 'Fluid Mind EmailHandler Class (PHP v' . phpversion() . ')';
            $headers['Mime-Version'] = '1.0';
            $headers['From'] = $this->from;
            if ($this->replyTo != '') {
                $headers['Reply-To'] = $this->replyTo;
            }
            if ($this->bounceTo != '') {
                $headers['Return-Path'] = $this->bounceTo;
                $headers['Errors-To'] = $this->bounceTo;
                $params = ' -f' . $this->bounceTo;
            }


            if ($this->body != '' and $this->htmlBody != '') {
                $isMultipartAlternative = true;
            } else {
                $isMultipartAlternative = false;
            }

            // Create the BODY part of the email

            // Will this message contain an HTML version with a plain-text alternative?
            if ($isMultipartAlternative) {
                // Create a unique string for the MIME boundaries
                $boundary = 'FM' . md5(uniqid('', true));

                $headers['Content-Type'] = "multipart/alternative; boundary=$boundary";

                $bodyPart = "This is a multi-part message in MIME format.\n";
                $bodyPart .= "--$boundary\n";
                $bodyPart .= "Content-Type: text/plain; charset={$this->encoding}; format=flowed\n";
                if ($this->encodeBody) {
                    $bodyPart .= "Content-Transfer-Encoding: base64\n\n";
                    $bodyPart .= chunk_split(base64_encode($this->body . "\n")) . "\n\n";
                } else {
                    $bodyPart .= $this->body . "\n\n";
                }
                $bodyPart .= "--$boundary\n";
                $bodyPart .= "Content-Type: text/html; charset={$this->encoding}\n";
                if ($this->encodeBody) {
                    $bodyPart .= "Content-Transfer-Encoding: base64\n";
                    $bodyPart .= "\n" . chunk_split(base64_encode($this->htmlBody . "\n")) . "\n\n";
                } else {
                    $bodyPart .= "\n" . $this->htmlBody . "\n\n";
                }
                $bodyPart .= "--$boundary--\n\n";
            } else {
                // Or will it be only HTML
                if ($this->htmlBody != '') {
                    $headers['Content-Type'] = "text/html; charset={$this->encoding};";
                    $bodyPart = ($this->encodeBody ? chunk_split(base64_encode($this->htmlBody . "\n")) : $this->htmlBody) . "\n";
                // Or only plain-text
                } else {
                    $headers['Content-Type'] = "text/plain; charset={$this->encoding}; format=flowed";
                    $bodyPart = ($this->encodeBody ? chunk_split(base64_encode($this->body . "\n")) : $this->body) . "\n";
                }
                if ($this->encodeBody) {
                    $headers['Content-Transfer-Encoding'] = 'base64';
                }
            }


            // Add any attachments
            if (sizeof($this->attachments) > 0) {
                // Create a unique string for the MIME boundaries between attachments
                $boundary = 'FM' . md5(uniqid('', true));

                // Save the content type and encoding of the body part, which had been set earlier as the content type of the whole message
                $bodyPartContentType = $headers['Content-Type'];
                $headers['Content-Type'] = "multipart/related; boundary=$boundary";
                $bodyPartEncoding = $headers['Content-Transfer-Encoding'];
                unset($headers['Content-Transfer-Encoding']);

                // Start building the actual message
                $body = "--$boundary\n";
                $body .= "Content-Type: $bodyPartContentType\n";
                if (!$isMultipartAlternative) {
                    $body .= "Content-Transfer-Encoding: $bodyPartEncoding\n";
                }
                $body .= "\n$bodyPart\n\n";

                // Add each attachment to the message body
                foreach ($this->attachments as $contentId => $filePath) {
                    if (file_exists($filePath)) {
                        $body .= "\n--$boundary\n";
                        $body .= "Content-Type: " . mime_content_type($filePath) . "; name=\"" . basename($filePath) . "\"\n";
                        $body .= "Content-ID: <" . $contentId . ">\n";
                        $body .= "Content-Transfer-Encoding: base64\n";
                        $body .= "Content-Disposition: inline; filename=\"" . basename($filePath) . "\"\n\n";
                        $body .= chunk_split(base64_encode(file_get_contents($filePath)));
                        $body .= "\n";
                    } else {
                        throw new Exception('Error getting attachment. "' . basename($filePath) . '" was not found.');
                    }
                }

                $body .= "--$boundary--\n";
            } else {
                $body = $bodyPart;
            }

            // Build the recipient headers
            $to = implode(', ', array_merge($recipient, $this->recipients));
            if (sizeof($this->ccRecipients) > 0) {
                $headers['Cc'] = implode(', ', $this->ccRecipients);
            }
            if (sizeof($this->bccRecipients) > 0) {
                $headers['Bcc'] = implode(', ', $this->bccRecipients);
            }

            // Build the mail headers
            $mailHeaders = '';
            foreach (array_merge($headers, $this->headers) as $key => $value) {
                $mailHeaders .= "$key: $value\n";
            }

            if ($subject == '') {
                $subject = $this->subject;
            }

            // EVENTUALLY I'LL ADD SUPPORT FOR SENDING THROUGH AN SMTP SERVER
            // FOR NOW, THIS WORKS IN LINUX, AND WILL WORK IN WINDOWS IF THE
            // SMTP SERVER ADDRESS IS SET IN PHP.INI
            // And off it goes!
            if (!(mail($to, mb_encode_mimeheader($subject, $this->encoding, 'Q'), $body, $mailHeaders))) {
                throw new Exception("There was a problem sending the email message.");
            }

        } else {
            if (sizeof($this->recipients) == 0 and $email == '') {

            } elseif($this->from == '') {
                $message = "Could not send message. Please specify the address this message will be sent from.";
            } elseif($this->body == '' and $this->htmlBody == '') {
                $message = "Could not send message. Please specify the address this message will be sent from.";
            }
            throw new Exception($message);
        }
    } // send()


    /**
     * Takes an email address and optional name and puts it into a valid
     * RFC 2822 address for a From, To, Cc or Bcc mail header
     * 
     * @param string $email An email address to send this message to
     * @param string $name (Optional) The name of the person corresponding to the given email address
     */
    private function buildEmailAddress($email = '', $name = '') {
        mb_internal_encoding($this->encoding);
        $result = '';
        $email = trim($email);
        $name = trim($name);

        if ($email != '') {
            if ($name == '') {
                $result = $email;
            } else {
                $result = '"' . mb_encode_mimeheader($name, $this->encoding, 'Q') . '" <' . $email . '>';
            }
        }

        return $result;
    }


}  // EmailHandler class

?>
